@@ -31,6 +31,32 @@ else:
     sqlalchemy_to_python_type = [(satype.__class__,pytype) for pytype,satype in sqlalchemy.types._type_map.iteritems()]
 sqlalchemy_to_python_type.append((sqlalchemy.types.String,str))
 
+def BoundPropertyAttribute(classMember, attribute):
+    """
+    Abstracts out property determination code between a UserAttr and SA column.
+
+    Attributes like the label, yoke and what's this string are kept in the 
+    SA info dictionary for a column.  They are kept directly as members of the 
+    descriptor class for a UserAttr.
+    """
+
+    # classMember appears like a SA column
+    if hasattr(classMember,"label") and callable(classMember.label):
+        col=classMember.property.columns[0]
+        if hasattr(col,"info") and col.info.has_key(attribute):
+            return col.info[attribute]
+
+    try:
+        # cls has an attribute col that looks like a UserAttr
+        return getattr(classMember, attribute)
+    except AttributeError:
+        pass
+
+    return None
+
+def ClassAttributeWhatsThis(classMember):
+    return BoundPropertyAttribute(classMember, "whats_this")
+
 def UserLabel(identifier):
     """
     Convert a python or SQL identifier name to a user viewable label.  Essentially this makes word breaks 
@@ -779,14 +805,7 @@ class PBTableModel(QtCore.QAbstractTableModel):
 
         return None
 
-    def setData(self, index, value, role=QtCore.Qt.EditRole):
-        #print value
-        if not index.isValid():
-            return None
-        #elif role != QtCore.Qt.DisplayRole and role != QtCore.Qt.EditRole:
-            #return None
-
-        # if this is a "flipper" row, then insert it as an official row
+    def ensureNoFlipper(self, index):
         if index.row() >= len(self.row_list):
             self.row_manager.join_flipper(self.flipper_rows[0])
             del self.flipper_rows[0]
@@ -795,9 +814,25 @@ class PBTableModel(QtCore.QAbstractTableModel):
                 self.flipper_rows.append(self.row_manager.add_flipper_row())
                 self.endInsertRows()
 
+    def setData(self, index, value, role=QtCore.Qt.EditRole):
+        #print value
+        if not index.isValid():
+            return None
+        #elif role != QtCore.Qt.DisplayRole and role != QtCore.Qt.EditRole:
+            #return None
+
         c = self.columns[index.column()]
         r = self.row_object(index.row())
-        setattr(r,c.attr if hasattr(c, "attr") else c,fromQType(value, attrType(self.cls, c)))
+        newValue = fromQType(value, attrType(self.cls, c))
+        oldValue = getattr(r, c.attr if hasattr(c, "attr") else c)
+        #print attrType(self.cls, c), newValue, oldValue
+        #  The None/"" dichotomy is a frightening mess -- we need to think carefully about all that.
+        if oldValue is None:
+            oldValue = toQType(oldValue, attrType(self.cls, c))
+        if newValue != oldValue:
+            self.ensureNoFlipper(index)
+
+            setattr(r, c.attr if hasattr(c, "attr") else c, newValue)
         return True
 
     def flags(self,index):
@@ -826,11 +861,18 @@ class WidgetAttributeMapper(object):
     function of WidgetAttributeMapper provides the ability to take a widget 
     created through other means and connect it to this mapping with a yoke.  
     This may be used with widgets generated via Qt designer tool.
+    
+    :param cls:  class we are mapping; we look for python descriptors in 
+        this class (both UserAttr and sqlalchemy properties)
+    :param objectConverter:  a callable taking an instance of type cls and 
+        returning an id/handle as appropriate for any bound CommandMenu 
+        entities.
     """
     
-    def __init__(self,cls,obj=None):
+    def __init__(self, cls, obj=None, objectConverter=None):
         self.cls = cls
         self.obj = obj
+        self.objectConverter = objectConverter
         self.widgets = []
         self.yokes = []
         self.attribute_map = {}
@@ -967,9 +1009,9 @@ class MapperMixin(object):
         if not hasattr(self,"_mapper_list"):
             self._mapper_list = []
 
-    def mapClass(self,cls):
+    def mapClass(self, cls, objectConverter=None):
         self.init_mapper_list()
-        m = WidgetAttributeMapper(cls)
+        m = WidgetAttributeMapper(cls, objectConverter=objectConverter)
         self._mapper_list.append(m)
         return m
 
