@@ -13,6 +13,7 @@ and sorting the commands for global application consistency.
 """
 
 import weakref
+import inspect
 from PyQtHelpers import *
 from PyQt4 import QtCore, QtGui
 
@@ -29,16 +30,18 @@ class Command(object):
 
     See :class:`BoundCommandMenu` for more details.
     """
-    def __init__(self, descr, default=False, requireSelection=True, needs_reload=True, key=None, statusTip=None, iconFile=None, type="action", sort=500):
+    def __init__(self, descr, default=False, viewRelated=False, requireSelection=True, needs_reload=True, key=None, statusTip=None, iconFile=None, type="action", sort=500):
         """
         :param descr: Text description of command for menu items
         :param default: Default command for item activation.
+        :param viewRelated: True if this command must be activated in the presence of a view
         :param requireSelection: Set to false if the command function makes sense with-out a current object
         :param needs_reload: Set to false if the command does not modify the database in a way that would require refreshing the screen.
         :param key: shortcut key as given by QtCore.Qt.Key enumeration
         :param type: classifies the command for menu structure - new/view/action/delete are common options with special meanings to the sort algorithm
         """
         self.descr = descr
+        self.viewRelated = viewRelated
         self.requireSelection = requireSelection
         self.default = default
         self.needs_reload = needs_reload
@@ -90,7 +93,7 @@ class CommandMenu(object):
         self.commands = []
         self.name = name
 
-    def command(self, descr, key=None, statusTip=None, type="action", iconFile=None, sort=500, default=False, requireSelection=False, needs_reload=True):
+    def command(self, descr, key=None, statusTip=None, type="action", iconFile=None, sort=500, default=False, requireSelection=False, viewRelated=False, needs_reload=True):
         """
         :param descr:  string that appears in the menu (or other UI element)
         :param key: short-cut key to apply to this command
@@ -110,6 +113,7 @@ class CommandMenu(object):
                 iconFile=iconFile, 
                 sort=sort, 
                 default=default, 
+                viewRelated=viewRelated, 
                 requireSelection=requireSelection, 
                 needs_reload=needs_reload)
             x.func = f
@@ -135,6 +139,8 @@ class CommandMenu(object):
             kwargs["iconFile"] = ":/qtalchemy/default-new.ico"
         if not kwargs.has_key("requireSelection"):
             kwargs["requireSelection"] = False
+        if not kwargs.has_key("viewRelated"):
+            kwargs["viewRelated"] = True
         kwargs["type"] = "new"
         return self.command(**kwargs)
 
@@ -153,6 +159,8 @@ class CommandMenu(object):
             kwargs["descr"] = "&Delete..."
         if not kwargs.has_key("requireSelection"):
             kwargs["requireSelection"] = True
+        if not kwargs.has_key("viewRelated"):
+            kwargs["viewRelated"] = True
         if not kwargs.has_key("iconFile"):
             kwargs["iconFile"] = ":/qtalchemy/default-delete.ico"
         kwargs["type"] = "delete"
@@ -171,6 +179,8 @@ class CommandMenu(object):
         """
         if not kwargs.has_key("requireSelection"):
             kwargs["requireSelection"] = True
+        if not kwargs.has_key("viewRelated"):
+            kwargs["viewRelated"] = True
         kwargs["type"] = "action"
         return self.command(descr, **kwargs)
 
@@ -214,6 +224,40 @@ class DomainEntity(CommandMenu):
         queryCols = tuple([self.key_column.label("_hidden_id")] + self.list_display_columns)
         return (Query(queryCols).order_by(self.list_display_columns[0], self.key_column), lambda x: x._hidden_id)
 
+
+class CommandEvent(object):
+    """
+    This class is passed with the `preCommand` signal, optionally to the command 
+    function, and to the `refresh` signal after the command function is executed.  
+    The command functions, decorated by members of :class:`CommandMenu`, are 
+    expected to indicate refresh context in a semi-standard way which isn't 
+    entirely known yet.
+    
+    Recipients of the `preCommand` signal can :func:`abort` the command if they 
+    wish; this is useful if the ambient screen could not save its data for 
+    some reason.  The CommandEvent class passed could be pre-loaded with settings 
+    which might be useful with-in the command function.
+    
+    Recipients of the `refresh` signal may possibly desire to reload from 
+    context clues put into the passed CommandEvent instance by the command 
+    function.
+    
+    Note that the free-form nature of screen context can create a tight coupling 
+    between the screens using the associate :class:`CommandMenu` and the the 
+    command functions themselves.
+    """
+    def __init__(self, cmd=None):
+        self.cmd = cmd
+        self.aborted = False
+    
+    def abort(self):
+        """
+        Call this method of the event parameter to the preCommand signal to 
+        stop the command function from being fired.  If aborted, the refresh 
+        signal will not be emitted.
+        """
+        self.aborted = True
+
 class BoundCommandMenuItem(object):
     """
     The class :class:`BoundCommandMenu` is responsible for creating and managing instances of this class.
@@ -254,27 +298,65 @@ class BoundCommandMenuItem(object):
             indexes = self.binding.view.selectedIndexes()
             self.action.setEnabled(len(indexes)>0)
 
+    def objectActivatedProxy(self, obj=None, objectConverter=None):
+        # self.objectConverter takes precedence!
+        objectConverter = objectConverter if self.objectConverter is None else self.objectConverter
+        event = CommandEvent(self.cmd)
+        self.binding.preCommand.emit(event)
+        if event.aborted:
+            return
+        
+        # this is a bit of a hack to preCommand first:
+        # it is presumed that preCommand saves the data so that the database keys are up-to-date
+        handle = obj
+        if handle is not None:
+            handle = objectConverter(handle)
+        args = inspect.getargspec(self.cmd.func).args
+        d = {}
+        if 'event' in args:
+            d['event'] = event
+        self.cmd.func(self.binding.target, handle, **d)
+        self.binding.refresh.emit(event)
+        if self.cmd.needs_reload:
+            if self.binding.view:
+                self.model.reset_content_from_session()
+
     def activatedProxy(self, index):
         if self.cmd.requireSelection and index is None:
             print "none selected"
         else:
-            if index is None:
-                self.cmd.func(self.binding.target)
+            if self.cmd.viewRelated:
+                handle = index
+                if handle is not None:
+                    handle = handle.internalPointer()
+                self.objectActivatedProxy(handle, self.model.objectConverter)
             else:
-                objectConverter = self.model.objectConverter if self.objectConverter is None else self.objectConverter
-                self.cmd.func(self.binding.target, objectConverter(index.internalPointer()))
-            if self.cmd.needs_reload:
-                self.binding.refresh.emit()
-                if self.binding.view:
-                    self.model.reset_content_from_session()
+                event = CommandEvent(self.cmd)
+                self.binding.preCommand.emit(event)
+                if event.aborted:
+                    return
+                try:
+                    args = inspect.getargspec(self.cmd.func).args
+                except Exception as e:
+                    args = ()
+                d = {}
+                if 'event' in args:
+                    d['event'] = event
+                self.cmd.func(self.binding.target, **d)
+                self.binding.refresh.emit(event)
 
     def triggeredProxy(self, checked=False):
-        index = None
-        if self.binding.view is not None:
+        if self.binding.obj is not None:
+            obj = self.binding.obj
+            self.objectActivatedProxy(obj)
+        elif self.binding.mapper is not None:
+            self.objectActivatedProxy(self.binding.mapper.obj, self.binding.mapper.objectConverter)
+        elif self.binding.view is not None:
             indexes = self.binding.view.selectedIndexes()
             index = None if len(indexes)==0 else indexes[0]
-        self.activatedProxy(index)
-
+            self.activatedProxy(index)
+        else:
+            self.activatedProxy(None)
 
 class BoundCommandMenu(QtCore.QObject):
     """
@@ -288,14 +370,16 @@ class BoundCommandMenu(QtCore.QObject):
     default context menu policy of :class:`qtalchemy.widgets.TableView` is 
     QtCore.Qt.ActionsContextMenu.
     """
-    preCommand = Signal(name='preCommand')
-    refresh = Signal(name='refresh')
+    preCommand = Signal(object, name='preCommand')
+    refresh = Signal(object, name='refresh')
 
-    def __init__(self, target, menu, view=None, buttonBox=None, bindDefault=False, objectConverter=None):
+    def __init__(self, target, menu, view=None, mapper=None, obj=None, buttonBox=None, bindDefault=False, objectConverter=None):
         QtCore.QObject.__init__(self)
         self.target = target
         self.menu = menu
         self.view = view
+        self.mapper = mapper
+        self.obj = obj
         self.buttonBox = buttonBox
         self.bindDefault = bindDefault
         self.objectConverter = objectConverter
@@ -311,7 +395,25 @@ class BoundCommandMenu(QtCore.QObject):
         
         :param view: a QAbstractItemView attached to a QAbstractItemModel
         """
-        return BoundCommandMenu(self.target, self.menu, view, buttonBox=buttonBox, bindDefault=bindDefault, objectConverter=objectConverter)
+        return BoundCommandMenu(self.target, self.menu, view=view, buttonBox=buttonBox, bindDefault=bindDefault, objectConverter=objectConverter)
+
+    def withMapper(self, mapper, buttonBox=None, bindDefault=False, objectConverter=None):
+        """
+        Return a duplicate of this BoundMenuCommand augmented with a view.  The 
+        QAction objects will enable and disable as appropriate for the connected view.
+        
+        :param view: a QAbstractItemView attached to a QAbstractItemModel
+        """
+        return BoundCommandMenu(self.target, self.menu, mapper=mapper, buttonBox=buttonBox, bindDefault=bindDefault, objectConverter=objectConverter)
+
+    def withObject(self, obj, buttonBox=None, objectConverter=None):
+        """
+        Return a duplicate of this BoundMenuCommand augmented with a view.  The 
+        QAction objects will enable and disable as appropriate for the connected view.
+        
+        :param view: a QAbstractItemView attached to a QAbstractItemModel
+        """
+        return BoundCommandMenu(self.target, self.menu, obj=obj, buttonBox=buttonBox, objectConverter=objectConverter)
 
     def resetActions(self):
         """
@@ -320,8 +422,15 @@ class BoundCommandMenu(QtCore.QObject):
         self.action_bindings = [] # this is annoying -- I need to hold a reference explicitly
         self.button_bindings = []
         self.default_binding = None
+
+        actionParent = self.view
+        if actionParent is None and isinstance(self.target, QtGui.QWidget):
+            actionParent = self.target
+        if actionParent is None:
+            actionParent = self.target.parent
+
         for cmd in self.menu.commands:
-            action = cmd.action(self.view if self.view is not None else self.target)
+            action = cmd.action(actionParent)
             current = BoundCommandMenuItem(cmd, action, objectConverter = self.objectConverter, binding = weakref.proxy(self))
             self.action_bindings.append(current)
             if cmd.default:
@@ -337,7 +446,8 @@ class BoundCommandMenu(QtCore.QObject):
 
         if self.view is not None:
             for cb in self.action_bindings:
-                self.view.addAction(cb.action)
+                if cb.cmd.viewRelated:
+                    self.view.addAction(cb.action)
 
     def fillMenu(self, menu):
         """
