@@ -24,6 +24,11 @@ def xdg_open(file):
     """
     import os
     if is_windows():
-        os.system('start {0}'.format(file))
+        try:
+            # we try with win32api because that's cleaner (os.system flickers a command prompt)
+            import win32api
+            win32api.ShellExecute(0, None, file, None, None, 1)
+        except ImportError as e:
+            os.system('start {0}'.format(file))
     else:
         os.system('xdg-open {0}'.format(file))
